import os
import sys

# 添加父目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from modules.bv2av import bv2av, av2bv

def main():
    aid = 115256957471455
    bvid = 'BV1YaJDzAEGP'
    print(f"av{aid} -> {av2bv(aid)}")
    print(f"{bvid} -> av{bv2av(bvid)}")

if __name__ == "__main__":
    main()



from typing import List, Literal
from pydantic import BaseModel, Field
from nonebot import get_plugin_config


class Config(BaseModel):
    """Plugin Config Here"""

    analysis_whitelist: list[str] = Field(default_factory=list)
    """B站解析用户白名单"""

    analysis_group_whitelist: list[str] = Field(default_factory=list)
    """B站解析群组白名单"""

    analysis_blacklist: list[str] = Field(default_factory=list)
    """B站解析用户黑名单"""

    analysis_group_blacklist: list[str] = Field(default_factory=list)
    """B站解析群组黑名单"""

    analysis_group_strategies: dict[str, List[
        Literal[
            "detail",
            "link",
            "comments",
        ]
    ]] = Field(default_factory=dict)
    """B站解析群组策略（detail、link、comments）"""

config = get_plugin_config(Config)



import aiohttp
from typing import Optional, Tuple
import urllib.parse

class ApiEncoder:
    'API 请求参数编码器'

    async def encode(self, url: str, params: dict) -> Tuple[str, dict]:
        '对请求参数进行编码，返回新的 url 和参数字典'
        return url, params

class ApiEnv:
    'API 请求环境参数'

    def __init__(self, ua: Optional[str], refer: Optional[str], cookie: Optional[str]):
        self.ua = ua or ''
        self.refer = refer or ''
        self.cookie = cookie or ''

    def get_encoder(self) -> Optional[ApiEncoder]:
        '获取请求参数编码器，默认不进行编码'
        return None

    def check_result(self, result: dict) -> Tuple[bool, str]:
        '检查返回结果是否成功，以及对应的错误信息'
        return False, str(result)

class ApiInfo:
    'API 请求接口信息'

    def __init__(
            self,
            url: str,
            params: dict,
            errors: dict,
            encoder: Optional[ApiEncoder]=None,
        ):
        self.url = url
        self.params = params
        self.errors = errors
        self.encoder = encoder

class ApiInvoker:
    'API 请求调用器'

    def __init__(self, env: ApiEnv, api_info: ApiInfo):
        self.env = env
        self.api_info = api_info

    async def call(self, headers: dict = {}):
        ua = self.env.ua
        refer = self.env.refer
        cookie = self.env.cookie
        api_info = self.api_info

        url = api_info.url
        params = api_info.params
        errors = api_info.errors
        encoder = api_info.encoder
        headers = {
            'User-Agent': ua,
            'Referer': refer,
            'Cookie': cookie,
            **headers
        }
        final_params = params

        if encoder:
            url, final_params = await encoder.encode(url, final_params)

        query = urllib.parse.urlencode(final_params)
        full_url = f'{url}?{query}'
        # print('Request URL:', full_url)
        # print('Request Headers:', headers)
        async with aiohttp.ClientSession() as session:
            async with session.get(full_url, headers=headers) as resp:
                resp.raise_for_status()
                json_content = await resp.json()
        # print('Response JSON:', json_content)
        [res_code, res_msg] = self.env.check_result(json_content)
        error_msg = errors.get(str(res_code), res_msg or '未知错误')
        if res_code != 0 and res_msg != '成功':
            raise Exception(f'API 请求失败，错误代码 {res_code}：{error_msg}')
        return json_content



XOR_CODE = 23442827791579
MASK_CODE = 2251799813685247
MAX_AID = 1 << 51
BASE = 58

DATA = 'FcwAPNKTMug3GV5Lj7EJnHpWsx4tb8haYeviqBz6rkCy12mUSDQX9RdoZf'


def av2bv(aid: int) -> str:
    """
    av号转 bv 号
    :param aid: av号整数（例如 170001）
    :return: 对应的 BV 字符串
    """
    bytes_list = list("BV1" + "0" * 9)  # 初始 12 个字符
    bv_index = len(bytes_list) - 1
    tmp = (MAX_AID | int(aid)) ^ XOR_CODE
    while tmp > 0:
        bytes_list[bv_index] = DATA[int(tmp % BASE)]
        tmp //= BASE
        bv_index -= 1
    # 交换指定位置
    bytes_list[3], bytes_list[9] = bytes_list[9], bytes_list[3]
    bytes_list[4], bytes_list[7] = bytes_list[7], bytes_list[4]
    return "".join(bytes_list)


def bv2av(bvid: str) -> int:
    """
    bv号转 av 号
    :param bvid: BV 字符串（例如 "BV1xx..."）
    :return: 对应的 av 整数
    """
    bvid_arr = list(bvid)
    # 先交换回去
    bvid_arr[3], bvid_arr[9] = bvid_arr[9], bvid_arr[3]
    bvid_arr[4], bvid_arr[7] = bvid_arr[7], bvid_arr[4]
    # 去掉前缀 "BV1"
    del bvid_arr[:3]
    tmp = 0
    for ch in bvid_arr:
        tmp = tmp * BASE + DATA.index(ch)
    return int((tmp & MASK_CODE) ^ XOR_CODE)



from nonebot.plugin import PluginMetadata, inherit_supported_adapters
from nonebot import require

require("nonebot_plugin_htmlrender")
require("nonebot_plugin_localstore")

from .config import Config
from .handlers import bili_helper

__plugin_meta__ = PluginMetadata(
    name="B站解析助手",
    description="主要功能：\n"
                "1. 提取分享小程序/图文卡片的视频链接\n"
                "2. 预览视频热评\n",
    usage="1. 分享B站小程序或图文卡片到群聊或私聊即可触发\n"
          "2. 管理员私聊发送“设置B站Cookie XXX”可设置请求的Cookie\n",
    config=Config,
    homepage="https://github.com/krimeshu/nonebot-plugin-bili-helper",
    type="application",
)
